//
// SlkThread.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package SLK;

/**
 * When a source thread tries to perform some action on a some
 * target thread, the following happens:
 * <ul>
 * <li> The source thread acquires the target thread's switch lock.
 * <li> The source thread checks to see if the access is allowed.
 *      (access is allowed iff the source thread can provide the
 *      ThreadRights object equal to the target thread's
 *      currentRights field).  If access is granted, then the source
 *      thread sets its targetThread field to point
 *      to the target thread.  If access is denied, targetThread
 *      remains null.
 * <li> The source thread performs the action (stop, suspend, etc.).
 *      This causes the target thread object's checkAccess to be called
 *      (we're still running in the source thread, though).
 * <li> checkAccess checks to see that the "this" thread object equals
 *      the source thread's targetThread field.  If they match, the
 *      action proceeds.  If not, an exception is thrown.
 * <li> The source thread sets its targetThread field to null.
 * <li> The source thread releases the target thread's switch lock.
 * </ul>
 *
 * <p> During this whole process, we have to worry about the source thread
 * itself getting killed.  In this case, targetThread needs to be
 * reset to null (in a finally clause), and the lock needs to be released.
 */

public class SlkThread extends Thread
{
    //
    // TODO: these two fields should not be public but simply default.
    // The problem is that the CallerCalleeProtected wrapper class,
    // which is in package w3c.jigsaw.slklet, cannot see them unless they are
    // public.
    // CallerCalleeProtected was supposed to extend an abstract class defined
    // in the package SLK. This abstract class mechanism requires the programmer
    // to define the abstract method, which is at best cumbersome.
    // As of now, make these fields public, and then think of a better solution.
    //

	public Object switchLock = new Object();
	public ThreadRights currentRights;
	SlkThread targetThread = null;

/*
        public SlkThread()
        public SlkThread(String name);
        public SlkThread(Runnable target);
        public SlkThread(Runnable target, String name);
        public SlkThread(ThreadGroup group, String name)
                throws SecurityException, IllegalThreadStateException;
        public SlkThread(ThreadGroup group, Runnable target)
                throws SecurityException, IllegalThreadStateException;
        public SlkThread(ThreadGroup group, Runnable target,
                        String name)
                throws SecurityException, IllegalThreadStateException;

        public SlkThread(ThreadRights threadRights)
                throws SecurityException;
        public SlkThread(ThreadRights threadRights, String name);
                throws SecurityException;
        public SlkThread(ThreadRights threadRights, Runnable target);
                throws SecurityException;
        public SlkThread(ThreadRights threadRights, Runnable target, String name);
                throws SecurityException;
        public SlkThread(ThreadRights threadRights, ThreadGroup group, String name)
                throws SecurityException, IllegalThreadStateException;
        public SlkThread(ThreadRights threadRights, ThreadGroup group, Runnable target)
                throws SecurityException, IllegalThreadStateException;
*/

   public SlkThread(ThreadRights threadRights, ThreadGroup group, Runnable target,
                    String name)
    throws SecurityException, IllegalThreadStateException
    {
      	super(group, target, name);
		threadRights.setThread(this);
		currentRights = threadRights;
	}

	public final void checkAccess()
	{
		try
		{
			SlkThread currentThread = (SlkThread) Thread.currentThread();
			if(this != currentThread.targetThread)
			{
				throw new SecurityException();
			}
		}
		catch(Exception e)
		{
			throw new SecurityException(e.getMessage());
		}
	}
}

